package org.rcisoft.openapi.api.accesscontrol;


import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.service.CyRedisService;
import org.rcisoft.core.util.CyAddressUtils;
import org.rcisoft.openapi.api.component.OpenApiGlobal;
import org.rcisoft.openapi.api.cons.OpenApiCons;
import org.rcisoft.openapi.api.redisconfig.RedisKeyDisposition;
import org.rcisoft.openapi.api.redisconfig.RedisSetKeyValue;
import org.rcisoft.openapi.iaminstitution.entity.IamInstitution;
import org.rcisoft.openapi.iaminstitution.service.IamInstitutionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@Component
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class AccessControl {

    @Autowired(required = false)
    CyRedisService cyRedisService;

    @Autowired
    OpenApiGlobal openApiGlobal;

    @Autowired
    IamInstitutionService iamInstitutionServiceImpl;

    @Autowired
    RedisSetKeyValue redisSetKeyValue;

    /**
     * OpenAPI全局拦截(ip)
     */
    public void ipGlobalInterceptor(HttpServletRequest request, HttpServletResponse response) {
        //ip
        String ip = CyAddressUtils.IpUtils.getIpAddr(request);
        //控制器
        Boolean illegal = false;
        //Ip访问不合法最大次数/分钟
        if (cyRedisService.hasKey(RedisKeyDisposition.OPEN_API_IP_ILLEGALLY_MIN.getKey() + ip)) {
            if ((Integer) cyRedisService.get(RedisKeyDisposition.OPEN_API_IP_ILLEGALLY_MIN.getKey() + ip) >= openApiGlobal.getIpIllegallyMin()) {
                illegal = true;
            }
        }
        //Ip访问不合法最大次数/天
        if (cyRedisService.hasKey(RedisKeyDisposition.OPEN_API_IP_ILLEGALLY_DAY.getKey() + ip)) {
            if ((Integer) cyRedisService.get(RedisKeyDisposition.OPEN_API_IP_ILLEGALLY_DAY.getKey() + ip) >= openApiGlobal.getIpIllegallyDay()) {
                illegal = true;
            }
        }
        if (illegal) {
            throw new CyServiceException(OpenApiCons.IP_LOCKED_OR_NOT_MATCH);
        }
        //控制器
        Boolean frequent = false;
        //Ip访问频繁访问最大次数/分钟
        if (cyRedisService.hasKey(RedisKeyDisposition.OPEN_API_IP_ACCESS_NUM_MIN.getKey() + ip)) {
            if ((Integer) cyRedisService.get(RedisKeyDisposition.OPEN_API_IP_ACCESS_NUM_MIN.getKey() + ip) >= openApiGlobal.getIpAccessNumMin()) {
                frequent = true;
            }
        }
        //Ip访问频繁访问最大次数/天
        if (cyRedisService.hasKey(RedisKeyDisposition.OPEN_API_IP_ACCESS_NUM_DAY.getKey() + ip)) {
            if ((Integer) cyRedisService.get(RedisKeyDisposition.OPEN_API_IP_ACCESS_NUM_DAY.getKey() + ip) >= openApiGlobal.getIpAccessNumDay()) {
                frequent = true;
            }
        }
        if (frequent) {
            throw new CyServiceException(OpenApiCons.IP_FREQUENTLY_ACCESS);
        }
        redisSetKeyValue.accessIsFrequently(request);


    }

    /**
     * OpenAPI全局拦截(ip login)
     */
    public void ipGlobalLoginInterceptor(HttpServletRequest request, HttpServletResponse response) {
        //ip
        String ip = CyAddressUtils.IpUtils.getIpAddr(request);
        //控制器
        Boolean illegal = false;
        //Ip访问不合法最大次数/分钟
        if (cyRedisService.hasKey(RedisKeyDisposition.OPEN_API_IP_ILLEGALLY_LOGIN_MIN.getKey() + ip)) {
            if ((Integer) cyRedisService.get(RedisKeyDisposition.OPEN_API_IP_ILLEGALLY_LOGIN_MIN.getKey() + ip) >= openApiGlobal.getIpIllegallyLoginMin()) {
                illegal = true;
            }
        }
        //Ip访问不合法最大次数/天
        if (cyRedisService.hasKey(RedisKeyDisposition.OPEN_API_IP_ILLEGALLY_LOGIN_DAY.getKey() + ip)) {
            if ((Integer) cyRedisService.get(RedisKeyDisposition.OPEN_API_IP_ILLEGALLY_LOGIN_DAY.getKey() + ip) >= openApiGlobal.getIpIllegallyLoginDay()) {
                illegal = true;
            }
        }
        if (illegal) {
            throw new CyServiceException(OpenApiCons.IP_LOCKED_OR_NOT_MATCH);
        }
        //控制器
        Boolean frequent = false;
        //Ip访问频繁访问最大次数/分钟
        if (cyRedisService.hasKey(RedisKeyDisposition.OPEN_API_IP_ACCESS_NUM_LOGIN_MIN.getKey() + ip)) {
            if ((Integer) cyRedisService.get(RedisKeyDisposition.OPEN_API_IP_ACCESS_NUM_LOGIN_MIN.getKey() + ip) >= openApiGlobal.getIpAccessNumLoginMin()) {
                frequent = true;
            }
        }
        //Ip访问频繁访问最大次数/天
        if (cyRedisService.hasKey(RedisKeyDisposition.OPEN_API_IP_ACCESS_NUM_LOGIN_DAY.getKey() + ip)) {
            if ((Integer) cyRedisService.get(RedisKeyDisposition.OPEN_API_IP_ACCESS_NUM_LOGIN_DAY.getKey() + ip) >= openApiGlobal.getIpAccessNumLoginDay()) {
                frequent = true;
            }
        }
        if (frequent) {
            throw new CyServiceException(OpenApiCons.IP_FREQUENTLY_ACCESS);
        }
        redisSetKeyValue.loginAccessIsFrequently(request);
    }

    /**
     * token验证
     */
    public void tokenInterceptor(HttpServletRequest request, HttpServletResponse response) {
        //token
        String token = request.getHeader("API_TOKEN");
        String redisToken = null;
        //redis中查询
        try {
            redisToken = openApiGlobal.getTokenHead() + cyRedisService.get(RedisKeyDisposition.OPEN_API_IP_GEI_TOKEN.getKey() + CyAddressUtils.IpUtils.getIpAddr(request)).toString();
        } catch (Exception e) {
            redisSetKeyValue.accessIsNotLegal(request);
            throw new CyServiceException(OpenApiCons.TOKEN_INVALID_OR_ERROR);
        }
        if (StringUtils.isEmpty(token) || !token.equals(redisToken)) {
            redisSetKeyValue.accessIsNotLegal(request);
            throw new CyServiceException(OpenApiCons.TOKEN_INVALID_OR_ERROR);
        }
    }

    /**
     * ip密钥查询
     */
    public IamInstitution ipSecretInterceptor(HttpServletRequest request, HttpServletResponse response) {
        //mysql 必开
        //ip
        String ip = CyAddressUtils.IpUtils.getIpAddr(request);
        //服务器开放ip
        IamInstitution byIp = iamInstitutionServiceImpl.findByIp(ip);
        if (byIp == null) {
            redisSetKeyValue.accessIsNotLegal(request);
            throw new CyServiceException(OpenApiCons.IP_LOCKED_OR_NOT_MATCH);
        }
        return byIp;
    }
}
