package org.rcisoft.openapi.api.log;

import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.AfterThrowing;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.rcisoft.openapi.openapilog.service.OpenApiLogService;
import org.springframework.beans.factory.annotation.Autowired;

import java.lang.reflect.Method;

/**
 * 操作日志记录处理
 *
 * @author isoft
 */
@Slf4j
@Aspect
public class ApiLogAsp {
    @Autowired
    OpenApiLogService openApiLogService;

    // 配置织入点
    @Pointcut("@annotation(org.rcisoft.openapi.api.log.ApiLogAnno)")
    public void logPointCut() {}

    /**
     * 处理完请求后执行
     *
     * @param joinPoint 切点
     */
    @AfterReturning(pointcut = "logPointCut()", returning = "jsonResult")
    public void doAfterReturning(JoinPoint joinPoint, Object jsonResult) {
        handleLog(joinPoint, null, jsonResult);
    }

    /**
     * 拦截异常操作
     *
     * @param joinPoint 切点
     * @param e         异常
     */
    @AfterThrowing(value = "logPointCut()", throwing = "e")
    public void doAfterThrowing(JoinPoint joinPoint, Exception e) {
        handleLog(joinPoint, e, null);
    }

    protected void handleLog(final JoinPoint joinPoint, final Exception e, Object jsonResult) {
        try {
            // 获得注解
            ApiLogAnno controllerLog = getAnnotationLog(joinPoint);
            if (controllerLog == null)
                return;
            log.info(String.valueOf(controllerLog));
            openApiLogService.keepALog(controllerLog, joinPoint, e, jsonResult);
        } catch (Exception exp) {
            // 记录本地异常日志
            log.error("==前置通知异常==" + exp.getClass().toString());
            log.error("异常信息:{}", exp.getMessage());

        }
    }


    /**
     * 是否存在注解，如果存在就获取
     */
    private ApiLogAnno getAnnotationLog(JoinPoint joinPoint) {
        Signature signature = joinPoint.getSignature();
        MethodSignature methodSignature = (MethodSignature) signature;
        Method method = methodSignature.getMethod();

        if (method != null) {
            return method.getAnnotation(ApiLogAnno.class);
        }
        return null;
    }

}
